# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._models_py3 import Error
from ._models_py3 import ErrorDetail
from ._models_py3 import InnerError
from ._models_py3 import Key
from ._models_py3 import KeyListResult
from ._models_py3 import KeyValue
from ._models_py3 import KeyValueFilter
from ._models_py3 import KeyValueListResult
from ._models_py3 import Label
from ._models_py3 import LabelListResult
from ._models_py3 import OperationDetails
from ._models_py3 import Snapshot
from ._models_py3 import SnapshotListResult
from ._models_py3 import SnapshotUpdateParameters

from ._azure_app_configuration_enums import CompositionType
from ._azure_app_configuration_enums import KeyValueFields
from ._azure_app_configuration_enums import LabelFields
from ._azure_app_configuration_enums import SnapshotFields
from ._azure_app_configuration_enums import SnapshotStatus
from ._azure_app_configuration_enums import State
from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "Error",
    "ErrorDetail",
    "InnerError",
    "Key",
    "KeyListResult",
    "KeyValue",
    "KeyValueFilter",
    "KeyValueListResult",
    "Label",
    "LabelListResult",
    "OperationDetails",
    "Snapshot",
    "SnapshotListResult",
    "SnapshotUpdateParameters",
    "CompositionType",
    "KeyValueFields",
    "LabelFields",
    "SnapshotFields",
    "SnapshotStatus",
    "State",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()
